/* Some constants for our code */
#define ETH_IP		0x800	; Ethernet type for IP 
#define IPT_TCP		6	; IP type for TCP
#define TCP_WWW		8080	; Destinatin port for WWW
#define FRAG_OFF	0x1FFF	; mask for offset portion of fragment bytes


/* We include a null initialization for microblock style */
#macro WWBumpInit[]
#endm



#macro WWBump[]
xbuf_alloc[$$hdr,6]			; Need 6 SDRAM registers

	/* First thing we do is set the output port of the frame.  Regardless */
	/* of classification, if it came in on 1, send it to 0 and visa versa */
	.local ifn
	  DL_GetInputPort[ifn]		; Get input port.  (in one of dl_reg*)
	  alu [ ifn, ifn, XOR, 1 ]	; XOR is much cheaper than branches
	  DL_SetOutputPort[ifn]		; Set output port.  (in one of dl_reg*)
	.endlocal


	/* Now we inspect the first 24 bytes of the frame header.  First we */
	/* get those 24 bytes from SDRAM */
	.local base off
	  Buf_GetData[base, dl_buffer_handle]		; Get the base SDRAM ptr
          DL_GetBufferOffset[off]			; pkt offset is in bytes
	  alu_shf[off, --, B, off, >>3]			; set off in Q-words
	  sdram[read, $$hdr0, base, off, 3], ctx_swap	; read 3 Q-words=6 regs
	.endlocal


	/* Now we check that the Ethernet type is IP */
	/* It should be in the 1st two bytes of the 4th longword */
	/* If its not an IP packet don't check any deeper */
	.local etype
	  immed[etype, ETH_IP]
	  alu_shf[ --, etype, -, $$hdr3, >>16]	; only 2nd operand is shifted
	  br!=0[NotWebSyn#]
	.endlocal


	/* Next we check to see if its a TCP packet.  The IP type field */
	/* should be in the last byte of the 6th longword.  If this is not */
	/* a TCP packet we are done checking it */
	br!=byte[$$hdr5, 0, IPT_TCP, NotWebSyn#]


	/* Now we know we have a TCP packet with at least 8 bytes of header */
	/* (we only need 4, but that's fine.  in here.  This next section */
	/* extracts the destination port and sees if its 80 */
	.local dpoff dport flags


	  .local base boff
	    /* First we want to get the length of the IP Header */
	    /* It should be in the third byte of the 4th longword */
	    /* that should be the 0x45XX.  So we shift it right 6 bits */
	    /* so the length is in the proper position and mask out the */
	    /* lower version and upper bits of the type of service field */
            ld_field_w_clr[dpoff, 0001, $$hdr3, >>6]	; shift before mask
            .local mask
              immed[mask, 0x3c]		; Mask out upper and lower 2 bits
              alu [ dpoff, dpoff, AND, mask ]
            .endlocal


	    /* Now dpoff contains the IP header length in bytes */
	    /* We want to get the quadword containing the TCP destination port*/
	    /* field.  So we add the ethernet header length and the offset */
	    /* of the destination port in the TCP header (2).  This gives us */
	    /* the offset from the beginning of the packet where we can find */
	    /* the destination port.   Next we get the base SDRAM address of */
	    /* the buffer.  We add the byte offset of the start of the frame */
	    /* to the byte offset of the TCP destination port.  We divide by 8*/
	    /* (shift right) to get the quad-word offset, and add that to the */
	    /* base.  Finally, we read that quadword */
            alu[dpoff, dpoff, +, 16]
	    Buf_GetData[base, dl_buffer_handle]
            DL_GetBufferOffset[boff]		
	    alu[boff, boff, +, dpoff]		; Add dport off to frame off
	    alu_shf[boff, --, B, boff, >>3]	; convert to Q-Words
	    sdram[read, $$hdr0, base, boff, 2], ctx_swap	; read 16 bytes

	  .endlocal


	  /* Now we don't know which of the two longwords the desgination */
	  /* port will be in.  However the IP header must be a multiple of 4 */
	  /* bytes in length so we know it will always be in the upper two */
	  /* bytes of the longword we select.  So get the lower 3 bits of the */
	  /* offset of the destination port and check whether that value is */
	  /* >= to 4.  If so, then the destination port is in the 2nd */
	  /* longword.  Otherwise it is in the first */
          alu[ dpoff, dpoff, AND, 0x7 ]
	  alu[ --, dpoff, -, 4]
	  br>=0[SecondWord#]

FirstWord#:

	  /* Either way we know that we need to load the upper 2 bytes */
	  /* of the longword to get the destination port */
	  ld_field_w_clr[dport, 0011, $$hdr0, >>16]	; shift before mask
	  ld_field_w_clr[flags, 0001, $$hdr2]
	  br[GotDstPort#]

SecondWord#:

	  /* Either way we know that we need to load the upper 2 bytes */
	  /* of the longword to get the destination port */
	  ld_field_w_clr[dport, 0011, $$hdr1, >>16]	; shift before mask
	  ld_field_w_clr[flags, 0001, $$hdr3]

GotDstPort#:

	  /* Finally, we have the destination port.  At this point we */
	  /* check to see if it is 80.  If not then again we are done with it */
	  .local wprt
	    immed[wprt, TCP_WWW]		; Check for port 80 
	    alu[--, dport, -, wprt]
	    br!=0[NotWebSyn#]
	  .endlocal


	  alu[--, flags, AND, 0x2]
	  br=0[NotWebSyn#]

	.endlocal


IsWeb#:

	.local exc
	  immed[exc, 0]
	  DL_SetExceptionCode[exc]
	.endlocal


	/* Set the TAG to our StrongARM component's TAG.  This also must be */
	/* done to raise an exception or else an error results. */
	.local ace_tag
	  immed32[ace_tag, WWBUMP_TAG]
	  DL_SetAceTag[ace_tag]
	.endlocal


	/* Finally set our return code to IX_EXCEPTION.  The dispatch loop */
	/* will actually raise the exception itself */
	immed[dl_next_block, IX_EXCEPTION]
	br[End#]

NotWebSyn#:

	/* If the packet we got is not a web packet then we just note that */
	/* the next microblock should process it.  dl_next_block is our */
	/* return code */
	immed32[dl_next_block, 1]

End#:

xbuf_free[$$hdr]
#endm
