/* action.c -- exception handler and default action for wwbump MicroACE */

#include <stdio.h>
#include <stdlib.h>
#include <wwbump.h>
#include <wwbcc.h>
#include <table.h>

#define TCP_FIN	0x1
#define TCP_SYN 0x2
#define TCP_RST	0x4
#define IPHOFF 14
#define IPSAOFF	12
#define IPDAOFF	16
#define TCPSPOFF 0
#define TCPDPOFF 2

ix_error getkey(ix_buffer b, struct hkey *hkp, int flags);

#define CK(x)	do { ix_error e;  e = (x);  \
		    if (e) { ix_error_dump(stderr, e); return e; } } while (0)


ix_error getkey(ix_buffer b, struct hkey *hkp, int flags)
{
  unsigned char *ihp, *thp;
  size_t len;
  int i;

  memset(hkp, 0, sizeof(hkp));

  CK(ix_buffer_get_size(b, &len));
  CK(ix_buffer_get_data(b, IPHOFF, len - IPHOFF, (char **)&ihp, &i));

  /* Skip to the TCP header */
  thp = ihp + ((*ihp & 0xf) << 2);

  if ( flags & TCP_RST )
  {
    memmove(&hkp->ip1, ihp + IPDAOFF, sizeof(hkp->ip1));
    memmove(&hkp->ip2, ihp + IPSAOFF, sizeof(hkp->ip2));
    memmove(&hkp->tcpp, thp + TCPDPOFF, sizeof(hkp->tcpp));
  }
  else
  {
    memmove(&hkp->ip1, ihp + IPSAOFF, sizeof(hkp->ip1));
    memmove(&hkp->ip2, ihp + IPDAOFF, sizeof(hkp->ip2));
    memmove(&hkp->tcpp, thp + TCPSPOFF, sizeof(hkp->tcpp));
  }

  return 0;
}



int exception(void *ctx, ix_ring r, ix_buffer b)
{
    struct wwbump *wwb = (struct wwbump *) ctx; 
    ix_error e;
    unsigned char flags;
    struct hkey k;
    (void) r;


    /* The exception code is the flags field of the packet */
    e = RmGetExceptionCode(wwb->tag, &flags);
    if ( e ) {
        fprintf(stderr, "%s:  Error getting exception code", 
                wwb->name);
        ix_error_dump(stderr, e);
        ix_buffer_del(b);
        return e;
    }


    /* Set up the key */
    getkey(b, &k, flags);

    if ( flags & TCP_SYN ) 
        markopen(&k);
    else
        markclosed(&k);


    /* Send the packet back down to our microblock group */
    /* our TAG is used to figure out an appropriate microengine */
    e = RmSendPacket(wwb->tag, b);
    if ( e ) {
        ix_error_dump(stderr, e);
        ix_buffer_del(b);
        return e;
    }


    return 0;
}





/* This function must be included.  It recieves frames from other    */
/* StrongARM components of other ACEs.  Since our only count path is */
/* from the microengines we delete all other packets sent to us      */
int ix_action_default(ix_ace * a, ix_buffer b)
{
    (void) a;		/* prevents warnings */
    ix_buffer_del(b);	/* Delete all frames */
    return RULE_DONE;	/* MUST do this */
}
