/* cc.c -- wwbump crosscall functions */

#include <stdlib.h>
#include <string.h>
#include <wwbump.h>
#include <wwbcc.h>
#include "wwbump_sk_c.h"
#include "wwbump_cc_c.h"


long Webcnt;		/* Stores the actual count of web packets */


ix_error cc_init(struct wwbump *wwb)
{
    CC_VMT_wwbump *vmt = 0;
    ix_cap *capp;
    ix_error e;

    /*
      Steps for setting up a crosscall server :
       - Initialize an ix_base_t structure to 0
       - Get the OMS communications access point (CAP) of the ACE
       - Call the crosscall initialization function
    */
    memset(&wwb->ccbase, 0, sizeof(wwb->ccbase));
    ix_ace_to_cap(&wwb->ace, &capp);
    e = sk_wwbump_init(&wwb->ccbase, capp);
    if (e) {
        free(wwb);
        return ix_error_new(0,0,e,"Error in sk_wwbump_init()\n");
    }

    /*
      Steps to retarget incomming crosscalls to our function instead
      of the default : 
       - Get a pointer to the CrossCall Virtual Method Table 
       - retarget the function pointers in that table to our functions
    */
    e = getCCVMT_wwbump(&wwb->ccbase, &vmt);
    if (e) 
        return ix_error_new(0,0,e,"Error in getCCVMT_wwbump()\n");
    vmt->_pCC_wwbump_getcnt = getcnt;

    /* Our counter of web connections */
    Webcnt = 0;

    inittab();

    return 0;
}


ix_error cc_fini(struct wwbump *wwb)
{
    ix_error e;
    /* Crosscall base finalization function */
    e = sk_wwbump_fini(&wwb->ccbase);
    if ( e )
        return ix_error_new(0,0,e,"Error in sk_wwbump_fini()\n");

    return 0;
}


/* This function is called every time someone makes a crosscall */
ix_error getcnt(ix_base_t* bp, long* rv)
{
    (void)bp;	/* prevents compiler warnings */

    /* Copy the web count into the return value */
    *rv = Webcnt;

    /* return 0 for success */
    return 0;
}
